//! Config parsing.

use std::{ffi::OsString, path::PathBuf};

/// Gets Nuarth's config directory.
fn get_config_dir() -> PathBuf {
    let env = std::env::var("XDG_CONFIG_HOME");
    if let Ok(env) = env {
        return PathBuf::from(env).join("nuarth");
    }
    #[allow(deprecated, clippy::unwrap_used)]
    std::env::home_dir().unwrap().join(".config").join("nuarth")
}

/// Configuration for tiling.
pub struct TilingConfig {
    /// The tiling factor used to calculate how many tiles in a screen.
    pub tiling_factor: f64
}

/// A keyboard key as represented by the linux kernel.
pub type Key = u32;

/// A keybind.
pub type Keybind = Vec<Key>;

/// Keybinds.
pub struct Keybinds {
    /// The keybind to change the tile selection left by one tile.
    pub select_left: Keybind,
    /// The keybind to change the tile selection right by one tile.
    pub select_right: Keybind,
    /// The keybind to change the tile selection up by one tile.
    pub select_up: Keybind,
    /// The keybind to change the tile selection down by one tile.
    pub select_down: Keybind,

    /// The keybind to expand the selected tile left by one tile.
    pub expand_left: Keybind,
    /// The keybind to expand the selected tile right by one tile.
    pub expand_right: Keybind,
    /// The keybind to expand the selected tile up by one tile.
    pub expand_up: Keybind,
    /// The keybind to expand the selected tile down by one tile.
    pub expand_down: Keybind,

    /// The keybind to move the selected tile left by one tile.
    pub move_left: Keybind,
    /// The keybind to move the selected tile right by one tile.
    pub move_right: Keybind,
    /// The keybind to move the selected tile up by one tile.
    pub move_up: Keybind,
    /// The keybind to move the selected tile down by one tile.
    pub move_down: Keybind,

    /// The keybind to change the workspace selection left by one workspace.
    pub select_workspace_left: Keybind,
    /// The keybind to change the workspace selection right by one workspace.
    pub select_workspace_right: Keybind,
    /// The keybind to change the workspace selection up by one workspace.
    pub select_workspace_up: Keybind,
    /// The keybind to change the workspace selection down by one workspace.
    pub select_workspace_down: Keybind,

    /// The keybind to move the selected workspace left by one workspace.
    pub move_workspace_left: Keybind,
    /// The keybind to move the selected workspace right by one workspace.
    pub move_workspace_right: Keybind,
    /// The keybind to move the selected workspace up by one workspace.
    pub move_workspace_up: Keybind,
    /// The keybind to move the selected workspace down by one workspace.
    pub move_workspace_down: Keybind,

    /// The keybind to bookmark the current workspace as number N where the keybind is prefix+N.
    pub bookmark_workspace_prefix: Keybind,

    /// The keybind to jump to bookmark number N where the keybind is prefix+N.
    pub jump_workspace_prefix: Keybind,
}

/// Nuarth configuration.
pub struct Config {
    /// Configuration for tiling.
    pub tiling: TilingConfig,
    /// Keybind configuration.
    pub keybinds: Keybinds,
    /// Environment variables.
    pub environment: Vec<(OsString, OsString)>
}

