//! Stuff related to state and dispatch methods.
#![warn(missing_docs, clippy::missing_docs_in_private_items, clippy::unwrap_used)]

use std::sync::RwLock;

use wayland_server::{
    protocol::{wl_surface::WlSurface, *}, Dispatch, Resource
};

/// The state of Nuarth.
pub struct DelegationState {
    /// Surfaces allocated.
    pub surfaces: Vec<wayland_server::protocol::wl_surface::WlSurface>,
}

/// The raw surface data. [Surface] is a thread-safe wrapper around this.
struct RawSurface {
    /// The parent of this surface.
    parent: Option<WlSurface>,
    /// The children of this surface.
    children: Vec<WlSurface>,
    /// The buffer used in this surface.
    buffer: Option<wl_buffer::WlBuffer>,
    /// The offset of the buffer.
    buffer_offset: (i32, i32)
}

/// A thread-safe wrapper around [RawSurface].
struct Surface(RwLock<RawSurface>);

/// The kind of a rectangle in a region
#[derive(Copy, Clone, Debug)]
enum RectangleKind {
    /// This rectangle should be added/unioned to the region
    Add,
    /// This rectangle should be removed/intersected with the region
    Remove,
}

/// A rectangle.
struct Rectangle<Coord> {
    /// The first point(generally top-left)
    point1: (Coord, Coord),
    /// The second point(generally bottom-right)
    point2: (Coord, Coord),
    /// The kind of this rectangle
    kind: RectangleKind,
}

/// The raw region data. [Region] is a thread-safe wrapper around this.
struct RawRegion {
    /// The shape of the region.
    shape: Vec<Rectangle<i32>>,
}

/// A thread-safe wrapper around [RawRegion].
struct Region(RwLock<RawRegion>);

impl Dispatch<wl_compositor::WlCompositor, ()> for DelegationState {
    fn request(
        state: &mut DelegationState,
        _client: &wayland_server::Client,
        _resource: &wl_compositor::WlCompositor,
        request: <wl_compositor::WlCompositor as wayland_server::Resource>::Request,
        _data: &(),
        _dhandle: &wayland_server::DisplayHandle,
        data_init: &mut wayland_server::DataInit<'_, DelegationState>,
    ) {
        match request {
            wl_compositor::Request::CreateSurface { id } => {
                println!("[TRACE] Creating new surface");

                let surface = data_init.init(
                    id,
                    Surface(RwLock::new(RawSurface {
                        parent: None,
                        children: vec![],
                        buffer: None,
                        buffer_offset: (0, 0)
                    })),
                );
                state.surfaces.push(surface);
            }
            wl_compositor::Request::CreateRegion { id } => {
                println!("[TRACE] Creating new region");

                data_init.init(id, Region(RwLock::new(RawRegion { shape: vec![] })));
            }
            unk => {
                println!("[ERROR] Unknown wl_compositor request {}!", unk.opcode());
            }
        }
    }
}

impl Dispatch<wl_surface::WlSurface, Surface> for DelegationState {
    fn request(
        state: &mut DelegationState,
        client: &wayland_server::Client,
        surface: &wl_surface::WlSurface,
        request: <wl_surface::WlSurface as wayland_server::Resource>::Request,
        data: &Surface,
        dhandle: &wayland_server::DisplayHandle,
        data_init: &mut wayland_server::DataInit<'_, DelegationState>,
    ) {
        match request {
            wl_surface::Request::Attach { buffer, x, y } => {
                let offset = (x != 0 || y != 0).then_some((x, y));
                
                let offset = if surface.version() < 5 {
                    offset
                } else {
                    surface.post_error(
                        wl_surface::Error::InvalidOffset,
                        "Passing non-zero x,y is protocol violation since version 5",
                    );
                    return;
                };

                {
                    #[allow(clippy::unwrap_used)] // That means the data has been poisoned and we want to panic too
                    let mut data = data.0.write().unwrap();
                    data.buffer = buffer;
                    data.buffer_offset = offset.unwrap_or((0,0));
                }
            }
            unk => {
                println!("[ERROR] Unknown wl_surface request {}!", unk.opcode());
            }
        }
    }
}

impl Dispatch<wl_region::WlRegion, Region> for DelegationState {
    fn request(
        state: &mut DelegationState,
        client: &wayland_server::Client,
        resource: &wl_region::WlRegion,
        request: <wl_region::WlRegion as wayland_server::Resource>::Request,
        data: &Region,
        dhandle: &wayland_server::DisplayHandle,
        data_init: &mut wayland_server::DataInit<'_, DelegationState>,
    ) {
    }
}
